/*  Hash Table Data Structure Example
    Copyright (C) 2001,2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* NOTE
 * The hash table copies the data you give it using a user defined
 * copy function. You must provide a copy function and hash function
 * for each data type.
 */


#ifndef CKE_PROJ_HASH_TABLE_H
#define CKE_PROJ_HASH_TABLE_H

/* glib is a portable library that contains many useful data
   structures. I use its singly linked list, GSList to avoid having
   to write one myself.
*/

#include <stdlib.h> /* size_t */
#include <glib.h> /* GList */

/* hash_func is used to hash data into the hash table. It takes two
 * arguments: data and sz. data is a void* to the data it must hash,
 * and sz is the size of the table. The hash_func must return a
 * number between 0 and sz-1. This number is the location in the hash
 * table that the data will be inserted into
 */
/* copy_func returns a copy of its argument */
/* eq_func takes two const void* args and returns 0 if the
 * data contained within them is equal
 */

typedef size_t (*hash_func) (void* data, size_t sz);
typedef void*  (*copy_func) (void* data);
typedef GCompareFunc eq_func;

struct _hash_table
{
  hash_func hash;
  copy_func copy;
  eq_func ecmp; /* equality comparison */
  size_t size; /* cannot be changed at run time */
  GSList** data; /* array of lists */
};

typedef struct _hash_table hash_table;

/* all functions take a hash_table*, even if they do not modify it to
   make the interface consistent
 */

/* make_hash_table
   takes 4 arguments: hash_func, copy_func, eq_func,  and size
   hash_func is a hash_func to hash the data with
   copy_func is the function used to copy data with
   eq_func is a function that determines if two values are equal
   size is the _static_ size of the hash table
*/
/* get and put get data from and put data into the table,
   respectively. get returns the data you are looking for if it is
   found (NULL if not found). put does not return any value 
*/
hash_table* make_hash_table    (hash_func, copy_func, eq_func, size_t);
void        destroy_hash_table (hash_table*); /* deletes hash_table */
void*       hash_table_get     (hash_table*, void*);
void        hash_table_put     (hash_table*, void*);
void        hash_table_remove  (hash_table*, void*);

#endif /* CKE_PROJ_HASH_TABLE_H */
