;    Hash Table Data Type - Example Library written in Scheme
;    Copyright (C) 2001  Clinton Ebadi
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

;;; implements a simple hash table
;;; note: Guile Scheme has hash tables built in
;;; I think that mine is nicer, but Guile's are implemented in C
;;; (I think), making it faster. This clobber the builtin hash
;;; table stuff.

(load "defstruct.scm") ; defstruct macro
(use-modules (srfi srfi-1)) ; list manipulation

; _hash-table data type (not for public use)
; members are hash, size, and table
; hash is the function used to hash data
; size is the size of the table
; table is the actual table
(defstruct _hash-table hash size table)

; make-hash-table
; takes 2 arguments: func and sz
; func: hash function, takes 2 args: data and size; returns an integer
;       that is less than size
; sz: size of table
(define make-hash-table
  (lambda (func sz)
    (define tbl (make-_hash-table))
    (set!_hash-table.hash tbl func)
    (set!_hash-table.size tbl sz)
    (set!_hash-table.table tbl (make-vector sz))
    (vector-fill! (_hash-table.table tbl) '()) ; fills table with null lists
    tbl)) ; returns tbl

; hash-table-retrieve
; takes 2 args: table and data
; searches table for data
; if found returns item searched for
; if not found, throws hash-not-found
(define hash-table-retrieve
  (lambda (table data)
    (let* ((temp-hash (_hash-table.hash table))
	   (location (temp-hash data (_hash-table.size table))))
      (if (null? (vector-ref (_hash-table.table table) location))
	  (throw 'hash-not-found) ; error, not in table
	  (begin ; time to search table
	    (let ((found (find ; next function is the equality predicate
			  (lambda (x) (equal? data x))
			   (vector-ref (_hash-table.table table) location))))
	      (if found
		  found ; return the item searched for
		  (throw 'hash-not-found) ; error, not in table
		  ))))))) ; lots of parens!

; hash-table-put!
; takes 2 args: table and data
; puts data into the hash table
; does not return a meaningful value
(define hash-table-put!
  (lambda (table data)
    (let* ((temp-hash (_hash-table.hash table))
	    (location (temp-hash data (_hash-table.size table))))
	 (vector-set! (_hash-table.table table) location
		      (append 
		       (vector-ref (_hash-table.table table) location)
		       (list data))))))

; cke-hash-table-remove!
; takes 2 args: table and data
; removes data from the hash table
; no useful return value
(define hash-table-remove!
  (lambda (table data)
    (let* ((temp-hash (_hash-table.hash table))
	   (location (temp-hash data (_hash-table.size table))))
	 (vector-set! (_hash-table.table table) location
		      (delete data
			      (vector-ref (_hash-table.table table) location)
			      )))))